/**************************************************************************************************
  Interrupts.c
**************************************************************************************************/

#include "Interrupts.h"
#include "Buffers.h"
#include "RGB.h"
#include "UART6.h"
#include "Timers.h"
#include "Loop_objects.h"


char IntFlag = 0;                 // Interrupt flag
char DetFlag = 0;                 // Detect flag

unsigned int Pass = 0;            // This variable counts the number of pulses from loop oscillator

unsigned int OldValue = 0;        // Previous Timer6 counter reading
unsigned int NewValue = 0;        // Current Timer6 counter reading
unsigned long Sum;                // The sum value of 4 most recent Timer6 counter readings
unsigned int Average;             // The average value of 4 most recent Timer6 counter readings
unsigned int LowThresh;           // Any value below this results in vehicle presence
unsigned int HighThresh;          // Any value above this results in vehicle absence

unsigned int ValueTable[4];       // Four most recent Timer6 counter values used in averaging
unsigned int InPtr;               // Pointer to the oldest entry in ValueTable

unsigned short LoopTimer;         // General purpose timer handle for this module
char LoopState = 1;               // Current state of loop state machine
char Tuned = 0;                   // Flag set after initial loop tuning


/**************************************************************************************************
   A stream of pulses from the loop detector is presented to the external interrupt on GPIO pin
   PE0. The rising edge of each pulse triggers the interrupt. The interrupt routine counts 100
   pulses before setting a flag. Processing of the interrupt is done in the background after the
   flag is set in order to allow other interrupts to take place.

   The free-running counter of Timer6 is read and the previous value of the counter is subtracted
   from it. The resulting value is placed into a 4 position buffer in a circular method. Each
   time a new value is inserted into the buffer, the 4 values are averaged. This smoothes out
   the values somewhat.

   Vehicle presence causes the loop oscillator to increase in frequency and the average value to
   decrease. When this value falls below a fixed threshold, vehicle presence is indicated.
   As the vehicle leaves the loop area, the oscillator frequency decreases and the Average
   value increases. Software hysteresis ensures that the vehicle is counted only once.

**************************************************************************************************/

/**************************************************************************************************
  External interrupt from slot 1 of the shield (PE0).
**************************************************************************************************/
void ExternalInterrupt0() iv IVT_INT_EXTI0 ics ICS_AUTO {

     PR0_bit = 1;                      // Clear interrupt

     if (++Pass > 99) {                // When counter overflows
         Pass = 0;                     // Start again at 0
         IntFlag = 1;                  // Set interrupt flag
         RGB_Set_Red();                // Diagnostic
     }
}

/**************************************************************************************************
  Enable external interrupts from slot1 of the shield (PE0).
**************************************************************************************************/
void ExtIntEnable(void) {

     // Configure PE0 as input with pull-up
     GPIO_Set_Pin_Mode(&GPIOE_BASE, _GPIO_PIN_0, _GPIO_CFG_MODE_INPUT | _GPIO_CFG_PULL_UP);

     // Set EXTI_IMR register
     MR0_bit = 1;                      // Unmask interrupt request from line 0

     // Set EXTI_RTSR register
     TR0_bit = 1;                      // Enable rising trigger

     // Set EXTI_FTSR register
     TR0_EXTI_FTSR_bit = 0;            // Disable falling trigger

     // Set SYSCFGEN bit in RCC_APB2ENR register
     SYSCFGEN_bit = 1;                 // Enable system configuration controller clock

     // Set SYSCFG_EXTICR1
     SYSCFG_EXTICR1 = 0x00000004;      // Connect PE0 to EXTI0

     NVIC_IntEnable(IVT_INT_EXTI0);    // Enable EXTI0 interrupt

     InPtr = 0;
}

/**************************************************************************************************
  Process the interrupt in the background when interrupt flag is set.
**************************************************************************************************/
void CheckPA0Int(void) {
     int i;

     if (IntFlag) {                    // If interrupt flag is set
         IntFlag = 0;                  // Clear interrupt flag
         RGB_Clr_Red();                // Diagnostic

         NewValue = ReadTimer6();      // Get the Timer6 free-running counter

         Sum = 0;                      // Initialize Sum prior to averaging calculation

         ValueTable[InPtr++] = NewValue - OldValue; // Subtract the old counter value from the new
                                                    // and put it into next position of buffer
                                                    // Increment buffer position pointer
         if (InPtr > 3) InPtr = 0;     // If buffer pointer exceeded buffer length set it to 0

         for (i=0;i<4;i++) {
              Sum += ValueTable[i];    // Sum the 4 most recent counter values
         }
         Average = (unsigned int)(Sum >> 2); // Divide by 4 to get average value

         OldValue = NewValue;          // Save most recent counter reading

         if (Tuned) {
             if ((Average < LowThresh) && (DetFlag == 0)) {  // If below this value
                 UART6_PutQueue("Vehicle present\r\n");      // indicate vehicle present
                 DetFlag = 1;                                // Flip detect flag
             }

             if ((Average > HighThresh) && (DetFlag == 1)) {  // If above this vlue
                 UART6_PutQueue("Vehicle absent\r\n");        // indicate vehicle absent
                 DetFlag = 0;                                 // Flip detect flag
             }
         }

     }
}

/**************************************************************************************************
  State machine for tuning the loop.
**************************************************************************************************/
void LoopSM(void) {
     int i;
     char *BufNum;
     
     switch (LoopState) {

         case 0: // Not used

             break;

         case 1: // Set timer to delay 500 ms
             LoopTimer = get_timer();            // Get a timer handle
             set_timer(LoopTimer,5000);           // Set timer
             UART6_PutQueue("Waiting for loop to stabilize. Please wait...\r\n");
             LoopState = 2;
             break;

         case 2: // Set loop thresholds
             if (check_timer(LoopTimer) == 0) {

                 UART6_PutQueue("Tuning loop\r\n");
                 
                 HighThresh = Average - UPPER;
                 LowThresh  = Average - LOWER;
/*
                 // Diagnostic
                 BufNum = GetBuffer();
                 sprintf(BufNum,"High threshold: %04X  Low threshold: %04X\r\n",HighThresh,LowThresh);
                 UART6_PutQueue(BufNum);
*/
                 Tuned = 1;
                 UART6_PutQueue("Ready\r\n");
                 set_timer(LoopTimer,60000);     // Set timer for 1 minute
                 LoopState = 3;
             }
             break;

         case 3: // Wait 1 minute. Do not proceed if vehicle is over the loop
             if ((check_timer(LoopTimer) == 0) && (DetFlag == 0)) {
                 UART6_PutQueue("Tuning loop\r\n");
                 LoopState = 4;
             }
             break;

         case 4: //  Tune loop and set timer for another minute
             if ((check_timer(LoopTimer) == 0) && (DetFlag == 0)) {
                 HighThresh = Average - UPPER;
                 LowThresh  = Average - LOWER;
/*
                 // Diagnostic
                 BufNum = GetBuffer();
                 sprintf(BufNum,"High threshold: %04X  Low threshold: %04X\r\n",HighThresh,LowThresh);
                 UART6_PutQueue(BufNum);
*/
                 set_timer(LoopTimer,60000);     // Set timer for 1 minute
                 LoopState = 3;
             }
             break;


     }
}











/**************************************************************************************************
  End of File.
**************************************************************************************************/