/**************************************************************************************************
  Timers.c
**************************************************************************************************/
#include "Timers.h"
#include "Common.h"
#include "UART6.h"
#include "Buffers.h"


// Variables
int TimerFlag = 0;  // Set by interrupt
unsigned int Msec = 0;




/*************************************************************************************************/
// Hardware Level Functions
/*************************************************************************************************/
/**************************************************************************************************
  System Timer Interrupt Handler */
void SystemTimerInterrupt() iv IVT_INT_SysTick ics ICS_AUTO {
     if (COUNTFLAG_bit) {              // Interrupts every millisecond
         irqhandler_timer();           // Service timer pool
         
         Msec += 1;                    // Increment millisecond counter
         if (Msec > 999) {             // When counter goes to 1000
             Msec = 0;                 // Set it to 0
             TimerFlag = 1;            // Set 1 second flag
         }
     }
}

/**************************************************************************************************
  Initialize system timer  */
void Timer_Init() {
     unsigned clk;
     char *BufNum;
     
     init_timerpool();                 // Make all timers free
     
     clk = Clock_Mhz();
     if (clk == 150)
         NVIC_SYSTICKRVR = 18750;      // Set reload value register for 150 MHz
     else if (clk == 120)
         NVIC_SYSTICKRVR = 15000;      // Set reload value register for 120 MHz
/*
     BufNum = GetBuffer();
     sprintf(BufNum,"NVIC_SYSTICKRVR: %d\r\n",NVIC_SYSTICKRVR);
     UART6_PutQueue(BufNum);
*/
     NVIC_SYSTICKCVR = 0;              // Reset timer counter
     ENABLE_SYSTICKCSR_bit = 1;        // Enable timer
     TICKINT_bit = 1;                  // Enable timer interrupt
}

/**************************************************************************************************
  Timer processing is done if timer flag is set */
void TimerCheck() {
     if (TimerFlag == 1) {
         TimerFlag = 0;                // Reset 1 second flag
         //UART6_PutQueue("Tick\r\n");
         //USB_PutQueue("Tick\r\n");
     }
}

//-------------------------------------------------------------------------------------------------




/*************************************************************************************************/
// Software Level Functions
/*************************************************************************************************/
/**************************************************************************************************
File:         Timers.c
Description:  Implements simple timer pool. No timer chaining
Resolution:   +0/-0.001 second

Author: Jari Lahti (jari.lahti@violasystems.com)
Version Info: 22.1.2004 - First version (JaL)
**************************************************************************************************/
//#include "timers.h"


// Variables
struct timers TimerPool[NUMTIMERS];


/**************************************************************************************************
Function:   init_timerpool
Parameters: none
Return val: void
Desc:       Initialize timer pool
**************************************************************************************************/
void init_timerpool(void) {
     unsigned short int i;

     for (i=0; i < NUMTIMERS; i++) {
          TimerPool[i].TimerField = 0;
          TimerPool[i].Free = TRUE;
     }
}

/**************************************************************************************************
Function:   get_timer
Parameters: none
Return val: unsigned short int - returns handle of free timer
Desc:       Gets free timer from the timer pool
**************************************************************************************************/
unsigned short int get_timer(void) {
     unsigned short int i;

     for (i=0; i < NUMTIMERS; i++) {
          if (TimerPool[i].Free == TRUE) {  // If a free timer is found
              TimerPool[i].Free = FALSE;    // Mark it reserved
              return i;                     // Return timer handle
          }
     }

     // Error Check
     UART6_PutQueue("TIMERS: No Timers, Resetting..\r\n");
     SystemReset();
     return (0);
}

/**************************************************************************************************
Function:   free_timer
Parameters: none
Return val: unsigned short int nbr - number
Desc:       Puts the given timer back to the timer pool
**************************************************************************************************/
void free_timer(unsigned short int nbr) {
     if (nbr > (NUMTIMERS-1))          // Make a simple check
         return;

     TimerPool[nbr].Free = TRUE;
}

/**************************************************************************************************
Function:   set_timer
Parameters: unsigned short int nbr - number
            unsigned long int tout - time out value
Return val: none
Desc:       Initializes given timer to given time-out
**************************************************************************************************/
void set_timer(unsigned short int nbr, unsigned long int tout) {
     if (nbr > (NUMTIMERS-1))          // Make a simple check
         return;

     if (TimerPool[nbr].Free == TRUE)
         return;

     TimerPool[nbr].TimerField = tout;
}

/**************************************************************************************************
Function:   check_timer
Parameters: unsigned short int nbr - number
Return val: unsigned long int - returns the value of given timer
Desc:       Return the value of given timer
**************************************************************************************************/
unsigned long int check_timer(unsigned short int nbr) {
     return TimerPool[nbr].TimerField;
}

/**************************************************************************************************
Function:   irqhandler_timer
Parameters: void
Return val: void
Desc:       Call this function on timer interrupt
**************************************************************************************************/
void irqhandler_timer(void) {
     unsigned short int i;

     for (i=0; i<NUMTIMERS; i++) {
          if (TimerPool[i].Free == FALSE && TimerPool[i].TimerField != 0)
          TimerPool[i].TimerField--;
     }
}







/**************************************************************************************************
   The counter clock frequency CK_CNT is equal to fCK_PSC / (PSC[15:0] + 1).
   The PSC register contains the value to be loaded into the active prescaler register
   at each update event.
   For a system clock of 120 MHz the fCK_PSC is 60,000,000. If (PSC[15:0] is 5 then
   CK_CNT is 60,000,000 / (5  + 1) = 10,000,000 Hz.
**************************************************************************************************/


/**************************************************************************************************
   Initialize Timer6
**************************************************************************************************/
void Timer6_Initialize(void) {
     
     TIM6EN_bit = 1;              // Enable TIM6 clock in RCC_APB1ENR
     TIM6_PSC = 5;                // Set the prescaler register to 5 (5 + 1 = divide by 6)
     
     CEN_TIM6_CR1_bit = 1;        // Enable counter in TIM6_CR1

}

/**************************************************************************************************
   Read Timer6 Free-Running Counter
**************************************************************************************************/
unsigned int ReadTimer6(void) {
     return TIM6_CNT;
}












/*************************************************************************************************/