/**************************************************************************************************
  UART6.c
  This UART module communicates with the FTDI Serial-to-USB FT232RL chip on the MikroMedia+ for
  STM32 shield. It's mainly used for diagnostic messages and requests for information.

  NOTE: ERRORS IN MIKROELEKTRONIKA SCHEMATIC!!!!!!!
  Net labeled HDR-UART2_TX connected to port PC6 physical pin 96 is actually USART6_TX
  Net labeled HDR-UART2_RX connected to port PC7 physical pin 97 is actually USART6_RX

  ***** 150 MHz calculation *****
  USART6 is on APB2 BUS and the peripheral clock is 75 MHz with a PLL running at 150 MHz
  USARTDIV = Fck / 16 * baud = 75,000,000 / 153,600 = 0d488.28125
  DIV_Mantissa = 0d488 = ox1E8
  DIV_Fraction = 16 * 0d0.28125 = 0d4.5  approximately 0x4
  Therefore USART6_BRR = 0x1E84

  ***** 120 MHz calculation *****
  Peripheral clock = 60 MHz with PLL running at 120 MHz
  USARTDIV = Fck / 16 * baud = 60,000,000 / 153,600 = 0d390.625
  DIV_Mantissa = 0d390 = 0x186
  DIV_Fraction = 16 * 0d0.625 = 0d10  approximately 0xA
  Therefore USART6_BRR = 0x186A

**************************************************************************************************/

// Include Files
#include "UART6.h"
#include "Buffers.h"
#include "RGB.h"


#pragma funcall UART6_Rx1, UART6_Rx2

// Variables
char RxIn;                   // Input pointer to the next byte in receive FIFO
char RxOut;                  // Output pointer to the next byte in receive FIFO
char RxQueue[rxqsize];       // Re-circulating receive FIFO

char RcvBuf[RcvBufSize];     // Receive messages are assembled in this buffer
char RcvBufPtr;              // Pointer to the next free byte in RcvBuf


char TxIn;                   // Input pointer next element in the transmit FIFO
char TxOut;                  // Output pointer to the next element in the transmit FIFO
char *TxQueue[txqsize];      // Queue of message pointers in the transmit FIFO

char *XmtBuf;                // Pointer to the current message being transmitted
char XmtBufPtr;              // Points to a byte in transmit buffer


// The receive message routine has 2 states
tPointerFcn UART6_RxFuncPtr = UART6_Rx1;  // Start in State 1

extern unsigned int Average;
extern unsigned long Sum;
extern char LoopState;
extern unsigned int LowThresh;
extern unsigned int HighThresh;



/*************************************************************************************************/
// Hardware Level Functions
/**************************************************************************************************
  Interrupt Handler */
void UART6_Interrupt() iv IVT_INT_USART6 ics ICS_AUTO {
     char R_data;        // Receive data byte
     char T_data;        // Transmit data byte


     if ((RXNE_USART6_SR_bit) /* || (ORE_bit) */ ) {
         R_data = USART6_DR;           // Fetch a byte from receive register
         RxQueue[RxIn++] = R_data;     // Put data byte in re-circulating buffer, increment pointer
         if (RxIn > (rxqsize-1))       // Adjust pointer if necessary
             RxIn = 0;
     }

     if ((TXE_USART6_SR_bit)&&(TXEIE_USART6_CR1_bit)) {
         T_data = XmtBuf[XmtBufPtr++]; // Get a byte from the transmit buffer
         if (T_data == NUL) {          // If it is the end of message marker
             TXEIE_USART6_CR1_bit = 0; // Disable transmitter interrupts
         } else {
             USART6_DR = T_data;       // Transfer data byte to transmit register
         }
     }
}

/**************************************************************************************************
  This routine sets the baud rate. */
void UART6_SetBaud(void) {

     // Library method of initializing the UART
     // The library uses the MCU Clock Frequency [MHz] found in in <Project, Edit Project...> to
     // set the correct baud rate. This field is not automatically updated when changing the
     // desired PLL frequency using the Load Scheme dialog. This value has to be manually changed
     // to match the loaded scheme.
     
     UART6_Init_Advanced(9600,_UART_8_BIT_DATA,_UART_NOPARITY,_UART_ONE_STOPBIT,
     &_GPIO_MODULE_USART6_PC67);

/*
     // Manual method of initializing the UART
     GPIOCEN_bit = 1;                  // Set GPIOC clock enable bit

     GPIOC_MODER |= 0x0000A000;        // Set port C MODER bits 6 and 7 to 10: alternate function mode
     GPIOC_AFRL |= 0x88000000;         // Select alternate function 1000: AF8 for bits 6 and 7

     USART6EN_bit = 1;                 // Set USART6 clock enable bit
     USART6_CR1 |= 0x0000200C;         // Set UE, TE, and RE in USART6 control register 1
     
     if (Clock_Mhz() == 150)
         USART6_BRR = 0x00001E84;      // Set baud rate register for 9600 with PLL at 150 MHz
     else if (Clock_Mhz() == 120)
        USART6_BRR = 0x0000186A;       // Set baud rate register for 9600 with PLL at 120 MHz
*/
     NVIC_IntEnable(IVT_INT_USART6);   // Enable interrupt vector (always required)

}

/**************************************************************************************************
  This routine starts the receiver. */
void UART6_StartRx(void) {
     RXNEIE_USART6_CR1_bit = 1;        // Interrupt when receive buffer is full (RXNE=1)
}

/**************************************************************************************************
  This routine starts the transmitter. */
void UART6_StartTx(void) {
     TXEIE_USART6_CR1_bit = 1;         // Interrupt when transmit buffer is empty (TXE=1)
}
//-------------------------------------------------------------------------------------------------



/*************************************************************************************************/
// UART6 Functions
/**************************************************************************************************
  Initialize UART */
void UART6_Initialize(void) {
     UART6_RxQueueInit();
     UART6_TxQueueInit();

     UART6_SetBaud();
     UART6_StartRx();
}

// The receive portion ****************************************************************************
/**************************************************************************************************
  This routine initializes the receive buffer in and out pointers. */
void UART6_RxQueueInit(void) {
     RxIn = RxOut = 0;                 // Set pointers to 0
     RcvBufPtr = 0;
}

/**************************************************************************************************
  This routine retrieves a byte from the re-circulating buffer if present. Returns 0 if no data
  available. Returns 1 if a data byte has been transferred successfully. The pointer to the data
  byte is passed to the routine. */
char UART6_GetChar(char *rxbyte) {
     if (RxIn != RxOut) {              // If there is a character in the buffer
        *rxbyte = RxQueue[RxOut++];    // Extract the character and update the pointer
        if (RxOut > (rxqsize-1))       // Check for wrap
           RxOut = 0;
        return 1;
     }else{
        return 0;                      // No data available
     }
}

/**************************************************************************************************
  UART_Rx State Machine. Call this function in the main loop. */
void UART6_Rx(void) {
     UART6_RxFuncPtr();    // Use the function pointer to go to a state
}

/**************************************************************************************************
  UART_Rx State 1: Accumulate received characters in a buffer until a Line Feed character is
  received then insert a NUL character into the buffer and go to State 2. */
void UART6_Rx1(void){
     char rxchar;

     if (UART6_GetChar(&rxchar)) {
         switch (rxchar) {
             case NUL:         // Discard NUL character
                 break;

             case '\r':        // Discard return character
                 break;

             case '\n':
                 RcvBuf[RcvBufPtr++] = rxchar;       // Put received character into buffer
                 RcvBuf[RcvBufPtr] = 0;              // End marker
                 RcvBufPtr = 0;                      // Point to start of buffer
                 UART6_RxFuncPtr = UART6_Rx2;        // Go to state 2
                 break;

             default:
                 if (RcvBufPtr < (RcvBufSize-2))     // Prevents buffer overrun
                     RcvBuf[RcvBufPtr++] = rxchar;   // Put character in buffer
                 break;
         }
     }
}

/**************************************************************************************************
  UART_Rx State 2: Execute the command indicated by the first byte in the buffer then return to
  State 1. */
void UART6_Rx2(void) {
     char *BufNum;
     unsigned clk;
     float Frequency;

     switch (RcvBuf[0]) {

        // Change to suit application

        case 'a': // Diagnostic
           UART6_PutQueue("Command 'a' received\r\n");
           break;

        case 'b': // Display Timer6 free-running counter
           BufNum = GetBuffer();
           sprintf(BufNum,"Timer6: %04X\r\n",TIM6_CNT);
           UART6_PutQueue(BufNum);
           break;

        case 'c': // Display the sum value
           BufNum = GetBuffer();
           sprintf(BufNum,"Sum value: %08LX\r\n",Sum);
           UART6_PutQueue(BufNum);
           break;

        case 'd': // Display the average value
           BufNum = GetBuffer();
           sprintf(BufNum,"Average value: %04X\r\n",Average);
           UART6_PutQueue(BufNum);
           break;

        case 'e': // Display the MPU PLL frequency
           clk = Clock_Mhz();
           BufNum = GetBuffer();
           sprintf(BufNum,"Clock: %d\r\n",clk);
           UART6_PutQueue(BufNum);
           break;

        case 'f': // Display the state machine state number
           BufNum = GetBuffer();
           sprintf(BufNum,"State: %d\r\n",LoopState);
           UART6_PutQueue(BufNum);
           break;

        case 'g': // Display assigned lower threshold
           BufNum = GetBuffer();
           sprintf(BufNum,"Lower threshold: %04X\r\n",LowThresh);
           UART6_PutQueue(BufNum);
           break;

        case 'h': // Display assigned upper threshold
           BufNum = GetBuffer();
           sprintf(BufNum,"Upper threshold: %04X\r\n",HighThresh);
           UART6_PutQueue(BufNum);
           break;

        case 'i': // Force-tune the loop and set thresholds
           LoopState = 1;
           break;

        case 'j': // Display frequency of loop oscillator
           Frequency = 1 / (Average * 0.000000001);
           BufNum = GetBuffer();
           sprintf(BufNum,"Loop oscillator frequency: %5.2f Hz\r\n",Frequency);
           UART6_PutQueue(BufNum);
           break;

        case 'k':
           break;

        case 'l':
           break;

        case 'm':
           break;

        case 'n':
           break;

        case 'o':
           break;

        case 'p':
           break;
        
        case 'r':
           break;

        case 's':
           break;

        case 't':
           break;

        case 'u':
           break;

        case 'v':
           break;

        case 'w':
           break;

        case 'x':
           break;

        case 'y':
           break;

        case 'z':
           break;


        default:
           UART6_PutQueue("Unknown command: ");

           BufNum = GetBuffer();                // Get a free buffer
           (void)sprintf(BufNum,"%c\r\n",RcvBuf[0]);
           UART6_PutQueue(BufNum);              // Send the message
           break;
     }
     UART6_RxFuncPtr = UART6_Rx1;               // Go back to state 1
}



// The transmit portion ***************************************************************************
/**************************************************************************************************
  This routine initializes the transmit buffer queue in and out pointers. */
void UART6_TxQueueInit(void) {
     TxIn = TxOut = 0;         // Set pointers to 0
     XmtBufPtr = 0;
}

/**************************************************************************************************
  This routine makes an entry into the transmit queue of pointers and updates the in pointer to
  the next available location. */
void UART6_PutQueue(char *msg) {
     TxQueue[TxIn++] = msg;    // Put the message pointer into FIFO
     if (TxIn > (txqsize-1))   // Adjust FIFO in pointer
         TxIn = 0;
}

/**************************************************************************************************
  This routine enters a pointer to a text message and starts the transmitter. */
void UART6_SendMsg(char *s) {
     XmtBuf = s;               // Pointer to the message for the interrupt handler
     XmtBufPtr = 0;            // Set character counter to start of text message
     UART6_StartTx();          // Turn on the transmitter
}

/**************************************************************************************************
  This routine checks the transmit buffer queue and if it is not empty, it extracts the pointer
  to the message and sets up the transmitter. Call this function in the main loop. */
void UART6_CheckQueue(void) {
     if (TXEIE_USART6_CR1_bit == 0) {   // Don't try to start transmitter if already transmitting
         if (TxIn != TxOut) {                 // If pointers are not the same
             UART6_SendMsg(TxQueue[TxOut++]); // Hand off the message pointer and increment counter
             if (TxOut > (txqsize-1))         // Check for wrap
                 TxOut = 0;
         }
     }
}